package rrdService.util;
import java.util.*;
import java.io.*;
import java.lang.*;
/**	
* The class DataSource allows to modelize a data source.
* @author	Damien Pellier
* @version	1.0
* date : 15/07/2001
* <dt><b>key-word</b><code>
* <dd>  RoundRobin
* </code>
* @see <a href="RoundRobinArchive.java">RoundRobinArchive</a>
* @see Lock LockRoundRobinDataBase LockRoundRobinLibrary
*/
public class DataSource implements Serializable {
	/**
	* GAUCE is for things like temperatures or number of people in a room or value of a RedHat share
	*/
	public static final String	GAUGE = "GAUGE";
	/**
	* COUNTER is for continuous incrementing counters like the InOctets counter in a router. The COUNTER data source assumes that the counter never decreases, except when a counter overflows. The update function takes the overflow into account. The counter is stored as a per-second rate. When the counter overflows, RRDtool checks if the overflow happened at the 32bit or 64bit border and acts accordingly by adding an appropriate value to the result. 
	*/
	public static final String	COUNTER = "COUNTER";
	/**
	* DERIVE will store the derivative of the line going from the last to the current value of the data source. This can be useful for gauges, for example, to measure the rate of people entering or leaving a room. Internally, derive works exaclty like COUNTER but without overflow checks. So if your counter does not reset at 32 or 64 bit you might want to use DERIVE and combine it with a MIN value of 0. 
	*/
	public static final String	DERIVE = "DERIVE";
	/**
	* ABSOLUTE is for counters which get reset upon reading. This is used for fast counters which tend to overflow. So instead of reading them normally you reset them after every read to make sure you have a maximal time available before the next overflow. Another usage is for things you count like number of messages since the last update. 
	*/
	public static final String	ABSOLUTE = "ABSOLUTE";
	/**
	* ds-Name is the name you will use to reference this particular data source from an RRD. A ds-name must be 1 to 19 characters long in the characters [a-zA-Z0-9_].
	*/
	private String dsName;
	/**
	* Data Source Type. The Datasource Type must be onw of the following: GAUGE, COUNTER, DERIVE, ABSOLUTE
	*/
	private String dataSourceType;
	/**
	* heartbeat defines the maximum number of seconds that may pass between two updates of this data source before the value of the data source is assumed to be *UNKNOWN*.
	*/
	private int heartbeat;
	/**
	* min defining the expected min range of the data supplied by this data source. 
	*/
	private double min;
	/**
	* max defining the expected max range of the data supplied by this data source. * 
	*/
	private double max;
	/**
	* Last data source
	*/
	private String lastDataSource;
	/**
	* value of this dataSource
	*/
	private double value;
	/**
	* Unknown second of this dataSource
	*/
	private int unknownSec;
	/**
	* Default constructor of a DataSource
	*/
	public DataSource() throws Exception {
	
		this.setDsName(new String());
		this.setDataSourceType(GAUGE);
		this.setHeartbeat(0);
		this.setMin(0);
		this.setMax(0);
		this.setLastDataSource("\"UNKN\"");
		this.setValue(0.0);
		this.setUnknownSec(0);
	}
	/**
	* Constructor of a DataSource
	* @param dsName the name of the data source
	* @param dataSourceType the type of the data source
	* @param heartBeat the heartbeat of the data source
	*/
	public DataSource(String dsName,
						String dataSourceType,
						int heartbeat) throws Exception {
	
		this.setDsName(dsName);
		this.setDataSourceType(dataSourceType);
		this.setHeartbeat(heartbeat);
		this.setMin(0);
		this.setMax(0);
		this.setLastDataSource("\"UNKN\"");
		this.setValue(0.0);
		this.setUnknownSec(0);
	}
	/**
	* Constructor of a DataSource
	* @param dsName the name of the data source
	* @param dataSourceType the type of the data source
	* @param heartBeat the heartbeat of the data source
	* @param min the min rang of the data source
	* @param max the max rang of the data source
	*/
	public DataSource(String dsName,
						String dataSourceType,
						int heartbeat,
						double min,
						double max) throws Exception {
	
		this.setDsName(dsName);
		this.setDataSourceType(dataSourceType);
		this.setHeartbeat(heartbeat);
		this.setMin(min);
		this.setMax(max);
		this.setLastDataSource("\"UNKN\"");
		this.setValue(0.0);
		this.setUnknownSec(0);
	}
	/**
	* Constructor of a DataSource
	* @param dsName the name of the data source
	* @param dataSourceType the type of the data source
	* @param heartBeat the heartbeat of the data source
	* @param min the min rang of the data source
	* @param max the max rang of the data source
	* @param lastDataSource the last data source of this data source
	* @param value the value of this data source
	* @param unknownSec the unknown second of this data source
	*/
	public DataSource(String dsName,
						String dataSourceType,
						int heartbeat,
						double min,
						double max,
						String lastDataSource,
						double value,
						int unknownSec) throws Exception {
	
		this.setDsName(dsName);
		this.setDataSourceType(dataSourceType);
		this.setHeartbeat(heartbeat);
		this.setMin(min);
		this.setMax(max);
		this.setLastDataSource(lastDataSource);
		this.setValue(value);
		this.setUnknownSec(unknownSec);
	}
	/**
	* Replaces the name in this data source
	* @param dsName the name in this data source to set
	*/
	public void setDsName(String dsName) {
		this.dsName = dsName.toUpperCase();
	}
	/**
	* Replaces the data source type in this data source
	* @param dataSourceType the data source type in this data source to set
	*/
	public void setDataSourceType(String dataSourceType) throws Exception {
		if (!isValideDataSourceType(dataSourceType))
			throw new Exception("Data source type not match");
		this.dataSourceType = dataSourceType.toUpperCase();
	}
	/**
	* isValideDataSourceType return true if the type of data source is valide
	*/
	public static boolean isValideDataSourceType(String dataSourceType) {
		return (dataSourceType.equalsIgnoreCase(GAUGE) ||
				dataSourceType.equalsIgnoreCase(COUNTER) ||
				dataSourceType.equalsIgnoreCase(DERIVE) ||
				dataSourceType.equalsIgnoreCase(ABSOLUTE)); 
	}
	/**
	* Replaces the heartbeat in this data source
	* @param heartbeat the heartbeat in this data source to set
	*/
	public void setHeartbeat(int heartbeat) {
		this.heartbeat = heartbeat;
	}
	/**
	* Replaces the min range in this data source
	* @param min the min range in this data source to set
	*/
	public void setMin(double min) {
		this.min = min;
	}
	/**
	* Replaces the max range in this data source
	* @param max the max range in this data source to set
	*/
	public void setMax(double max) {
		this.max = max;
	}
	/**
	* Replaces the lastDataSource in this data source
	* @param lastDataSource the last data source in this data source to set
	*/
	public void setLastDataSource(String lastDataSource) {
		this.lastDataSource = lastDataSource.toUpperCase();
	}
	/**
	* Replaces the value in this data source
	* @param value the value in this data source to set
	*/
	public void setValue(double value) {
		this.value = value;
	}
	/**
	* Replaces the unknownSec in this data source
	* @param unknownSec the unknownSec in this data source to set
	*/
	public void setUnknownSec(int unknownSec) {
		this.unknownSec = unknownSec;
	}
	/**
	* Returns the data source name of this data source.
	*/
	public String getDsName() {
		return this.dsName;
	}
	/**
	* Returns the data source type of this data source.
	*/
	public String getDataSourceType() {
		return this.dataSourceType;
	}
	/**
	* Returns the heartbeat of this data source.
	*/
	public int getHeartbeat() {
		return this.heartbeat;
	}
	/**
	* Returns the min range of this data source.
	*/
	public double getMin() {
		return this.min;
	}
	/**
	* Returns the max range of this data source.
	*/
	public double getMax() {
		return this.max;
	}
	/**
	* Return the lastDataSource in this data source
	*/
	public String getLastDataSource() {
		return this.lastDataSource;
	}
	/**
	* Return the value in this data source
	*/
	public double getValue() {
		return this.value;
	}
	/**
	* Return the unknownSec in this data source
	*/
	public int getUnknownSec() {
		return this.unknownSec;
	}
	/**
	* toString return a string representation of this data source for the command line
	*/
	public String toString() {
		return "DS:"+dsName+":"+dataSourceType+":"+String.valueOf(heartbeat)+":"+String.valueOf(min)+":"+String.valueOf(max);
	}
	/**
	* toStringAll return a string representation of this data source.
	*/
	public String toStringAll() {
		return 	"DS["+dsName+"].type = "+dataSourceType+"\n"+
				"DS["+dsName+"].minimal_heartbeat = "+String.valueOf(heartbeat)+"\n"+
				"DS["+dsName+"].min = "+String.valueOf(min)+"\n"+
				"DS["+dsName+"].max = "+String.valueOf(max)+"\n"+
				"DS["+dsName+"].last_ds = "+lastDataSource+"\n"+
				"DS["+dsName+"].value = "+String.valueOf(value)+"\n"+
				"DS["+dsName+"].unknown_sec = "+String.valueOf(unknownSec);
	}
}