package rrdService.util;
import rrdService.service.*;
import java.util.*;
import java.io.*;
import java.lang.*;
/**	
* The class MathExpressionVirtualDataSource 
* Create a new virtual data source by evaluating a mathematical expression,
* specified in Reverse Polish Notation
* <FONT SIZE="-1">(RPN).</FONT> If you have ever used a traditional
* <FONT SIZE="-1">HP</FONT> calculator you already know
* <FONT SIZE="-1">RPN.</FONT> The idea behind
* <FONT SIZE="-1">RPN</FONT> notation is,
* that you have a stack and push your data onto this stack. When ever
* you execute an operation, it takes as many data values from the stack
* as needed. The pushing of data is implicit, so when ever you specify a number
* or a variable, it gets pushed automatically.
* <P>If this is all a big load of incomprehensible words for you, maybe an
* example helps (a more complete explanation is given in [1]): The
* expression <EM>vname+3/2</EM> becomes <CODE>vname,3,2,/,+</CODE> in
* <FONT SIZE="-1">RPN.</FONT> First the three
* values get pushed onto the stack (which now contains (the current
* value of) vname, a 3 and a 2). Then the / operator pops two values
* from the stack (3 and 2), divides the first argument by the second
* (3/2) and pushes the result (1.5) back onto the stack. Then the +
* operator pops two values (vname and 1.5) from the stack; both values
* are added up and the result gets pushes back onto the stack. In the
* end there is only one value left on the stack: The result of the
* expression.</P>
* <P>The <EM>rpn-expression</EM> in the <STRONG>
* <FONT SIZE="-1">CDEF</FONT></STRONG> function takes both, constant values
* as well as <EM>vname</EM> variables. The following operators can be used on these
* values:</P>
* <DL>
* <DT><STRONG><A NAME="item_%2B%2C_%2D%2C_%2A%2C_%2F%2C_%25">+, -, *, /, %</A></STRONG><BR>
* <DD>
* pops two values from the stack applies the selected operator and pushes
* the result back onto the stack. The % operator stands for the modulo
* operation.
* <P></P>
* <DT><STRONG><A NAME="item_SIN%2C_COS%2C_LOG%2C_EXP%2C_FLOOR%2C_CEIL">
* <FONT SIZE="-1">SIN,</FONT>
* <FONT SIZE="-1">COS,</FONT>
* <FONT SIZE="-1">LOG,</FONT>
* <FONT SIZE="-1">EXP,</FONT>
* <FONT SIZE="-1">FLOOR,</FONT>
* <FONT SIZE="-1">CEIL</FONT></A></STRONG><BR>
* <DD>
* pops one value from the stack, applies the selected function and pushes
* the result back onto the stack.
* <P></P>
* <DT><STRONG><A NAME="item_LT%2C_LE%2C_GT%2C_GE%2C_EQ">
* <FONT SIZE="-1">LT,</FONT>
* <FONT SIZE="-1">LE,</FONT>
* <FONT SIZE="-1">GT,</FONT>
* <FONT SIZE="-1">GE,</FONT>
* <FONT SIZE="-1">EQ</FONT></A></STRONG><BR>
* <DD>
* pops two values from the stack, compares them according to the selected
* condition and pushes either 1 back onto the stack if the condition is true
* and 0 if the condition was not true.
* <P></P>
* <DT><STRONG><A NAME="item_IF">
* <FONT SIZE="-1">IF</FONT></A></STRONG><BR>
* <DD>
* pops three values from the stack. If the last value is not 0, the
* second value will be pushed back onto the stack, otherwise the
* first value is pushed back.
* <P>If the stack contains the values
* <FONT SIZE="-1">A,</FONT>
* <FONT SIZE="-1">B,</FONT>
* <FONT SIZE="-1">C,</FONT>
* <FONT SIZE="-1">D,</FONT>
* <FONT SIZE="-1">E</FONT> are presently on the
* stack, the
* <FONT SIZE="-1">IF</FONT> operator will pop the values
* <FONT SIZE="-1">E</FONT>
* <FONT SIZE="-1">D</FONT> and
* <FONT SIZE="-1">C</FONT> of the stack. It will
* look at
* <FONT SIZE="-1">C</FONT> and if it is not 0 it will push
* <FONT SIZE="-1">D</FONT> back onto the stack, otherwise
* <FONT SIZE="-1">E</FONT> will be sent back to the stack.</P>
* <P></P>
* <DT><STRONG><A NAME="item_MIN%2C_MAX">
* <FONT SIZE="-1">MIN,</FONT>
* <FONT SIZE="-1">MAX</FONT></A></STRONG><BR>
* <DD>
* selects the lesser or larger of the two top stack values respectively
* <P></P>
* <DT><STRONG><A NAME="item_LIMIT">
* <FONT SIZE="-1">LIMIT</FONT></A></STRONG><BR>
* <DD>
* replaces the value with <EM>
* <FONT SIZE="-1">*UNKNOWN*</FONT></EM> if it is outside the limits specified
* by the two values above it on the stack.
* <PRE>
*  CDEF:a=alpha,0,100,LIMIT</PRE>
* <P></P>
* <DT><STRONG><A NAME="item_DUP%2C_EXC%2C_POP">
* <FONT SIZE="-1">DUP,</FONT>
* <FONT SIZE="-1">EXC,</FONT>
* <FONT SIZE="-1">POP</FONT></A></STRONG><BR>
* <DD>
* These manipulate the stack directly.
* <FONT SIZE="-1">DUP</FONT> will duplicate the top of the
* stack, pushing the result back onto the stack.
* <FONT SIZE="-1">EXC</FONT> will exchange the top
* two elements of the stack, and
* <FONT SIZE="-1">POP</FONT> will pop off the top element of the
* stack. Having insufficient elements on the stack for these operations is
* an error.
* <P></P>
* <DT><STRONG><A NAME="item_UN">
* <FONT SIZE="-1">UN</FONT></A></STRONG><BR>
* <DD>
* Pops one value off the stack, if it is <EM>
* <FONT SIZE="-1">*UNKNOWN*</FONT></EM>, 1 will be pushed
* back otherwise 0.
* <P></P>
* <DT><STRONG><A NAME="item_UNKN">
* <FONT SIZE="-1">UNKN</FONT></A></STRONG><BR>
* <DD>
* Push an <EM>
* <FONT SIZE="-1">*UNKNOWN*</FONT></EM> value onto the stack.
* <P></P>
* <DT><STRONG><A NAME="item_PREV">
* <FONT SIZE="-1">PREV</FONT></A></STRONG><BR>
* <DD>
* Push <EM>
* <FONT SIZE="-1">*UNKNOWN*</FONT></EM> if its at the first value of a data set or otherwise
* the value of this
* <FONT SIZE="-1">CDEF</FONT> at the previous time step. This allows you to
* perform calculations across the data.
* <P></P>
* <DT><STRONG><A NAME="item_INF%2C_NEGINF">
* <FONT SIZE="-1">INF,</FONT>
* <FONT SIZE="-1">NEGINF</FONT></A></STRONG><BR>
* <DD>
* Push a positive or negative infinite (oo) value onto the stack. When
* drawing an infinite number it appears right at the top or bottom edge of the
* graph, depending whether you have a positive or negative infinite number.
* <P></P>
* <DT><STRONG><A NAME="item_NOW">
* <FONT SIZE="-1">NOW</FONT></A></STRONG><BR>
* <DD>
* Push the current (real world) time onto the stack.
* <P></P>
* <DT><STRONG><A NAME="item_TIME">
* <FONT SIZE="-1">TIME</FONT></A></STRONG><BR>
* <DD>
* Push the time the current sample was taken onto the stack. This is the
* number of non-skip seconds since 0:00:00 January 1, 1970.
* <P></P>
* <DT><STRONG><A NAME="item_LTIME">
* <FONT SIZE="-1">LTIME</FONT></A></STRONG><BR>
* <DD>
* This is like
* <FONT SIZE="-1">TIME</FONT> <STRONG>+ current timezone offset in seconds</STRONG>. The current
* offset takes daylight saving time into account, given your
* <FONT SIZE="-1">OS</FONT> supports
* this. If you were looking at a sample, in Zurich, in summer, the
* offset would be 2*3600 seconds, as Zurich at that time of year is 2
* hours ahead of
* <FONT SIZE="-1">UTC.</FONT>
* <P>Note that the timezone offset is always calculated for the time the
* current sample was taken at. It has nuthing todo with the time you are
* doing the calculation.</P>
* <P></P></DL>
* <P>Please note that you may only use <EM>vname</EM> variables that you
* previously defined by either <STRONG>
* <FONT SIZE="-1">DEF</FONT></STRONG> or <STRONG>
* <FONT SIZE="-1">CDEF</FONT></STRONG>. Furthermore, as of
* this writing (version 0.99.25), you must use at least one <EM>vname</EM>
* per expression, that is ``CDEF:fourtytwo=2,40,+'' will yield an error
* message but not a <EM>vname</EM> fourtytwo that's always equal to 42.</P>
* @author	Damien Pellier
* @version	1.0
* date : 15/07/2001
* </code>
* @see <a href="MathExpressionVirtualDataSource.java">MathExpressionVirtualDataSource</a>
* @see RoundRobinGraphic 
*/
public class MathExpressionVirtualDataSource implements Serializable {
	/**
	* virtualName virtual name for a data source
	*/
	private String virtualName;
	/**
	* rpnExpression of this virtual data source
	*/
	private String rpnExpression;
	/**
	* Constructor of MathExpressionVirtualDataSource
	* @param virtualName the virtual name for a data source
	* @param rpnExpression the rpnExpression of this virtual data source
	*/
	public MathExpressionVirtualDataSource(String virtualName, String rpnExpression) {
		setVirtualName(virtualName);
		setRpnExpression(rpnExpression);
	}
	/**
	* Replaces the the virtual name for a data source
	* @param virtualName the virtual name for a data source to set
	*/
	public void setVirtualName(String virtualName) {
		this.virtualName = virtualName;
	}
	/**
	* Replaces the rpnExpression of this virtual data source
	* @param rpnExpression the rpnExpression of this virtual data source to set
	*/
	public void setRpnExpression(String rpnExpression) {
		this.rpnExpression = rpnExpression;
	}
	/**
	* Returns the the virtual name for a data source
	*/
	public String getVirtualName() {
		return this.virtualName;
	}
	/**
	* Returns the rpnExpression of this virtual data source
	*/
	public String getRpnExpression() {
		return this.rpnExpression;
	}
	/**
	* Returns a string representation of this virtual data source built from a rpn expression
	*/
	public String toString() {
		return "CDEF:"+virtualName+"="+rpnExpression;
	}
}