package rrdService.util;
import java.util.*;
import java.io.*;
import java.lang.*;
/**	
* The class RoundRobinArchive consists of a number of data values from all the defined data-sources and is defined with an RRA line. 
* @author	Damien Pellier
* @version	1.0
* date : 15/07/2001
* <dt><b>key-word</b><code>
* <dd>  RoundRobin
* </code>
* @see <a href="RoundRobinArchive.java">RoundRobinArchive</a>
* @see Lock LockRoundRobinDataBase LockRoundRobinLibrary
*/
public class RoundRobinArchive implements Serializable {
	/**
	* The consolidation function of the archive. The following consolidation functions are defined: AVERAGE, MIN, MAX, LAST.
	*/
	private String consolidationFunction;
	/**
	* The xfiles factor defines what part of a consolidation interval may be made up from *UNKNOWN* data while the consolidated value is still regarded as known.
	*/
	private double xFilesFactor;
	/**
	* steps defines how many of these primary data points are used to build a consolidated data point which then goes into the archive.
	*/
	private int steps;
	/**
	* rows defines how many generations of data values are kept in an RRA.
	*/
	private int rows;
	/**
	* CdpPrepContainer of this RRA
	*/
	private CdpPrepContainer cdpPrepContainer;
	/**
	* Default constructor of a RoundRobinArchive
	*/
	public RoundRobinArchive() throws Exception {
		this.setConsolidationFunction(new String());
		this.setXFilesFactor(0);
		this.setSteps(0);
		this.setRows(0);
		this.setCdpPrepContainer(new CdpPrepContainer());
	}
	/**
	* Constructor of a RoundRobinArchive
	* @param consolidationFunction the consolidation function of the archive
	* @param xFilesFactor the xFilesFactor of the archive
	* @param steps the steps of the archive
	* @param rows the rows of the archive
	*/
	public RoundRobinArchive(String consolidationFunction,
								double xFilesFactor,
								int steps,
								int rows) throws Exception {
	
		this.setConsolidationFunction(consolidationFunction);
		this.setXFilesFactor(xFilesFactor);
		this.setSteps(steps);
		this.setRows(rows);
		this.setCdpPrepContainer(new CdpPrepContainer());
	}
	/**
	* Constructor of a RoundRobinArchive
	* @param consolidationFunction the consolidation function of the archive
	* @param xFilesFactor the xFilesFactor of the archive
	* @param steps the steps of the archive
	* @param rows the rows of the archive
	* @param cdpPrepContainer the CdpPrepContainer of the archive
	*/
	public RoundRobinArchive(String consolidationFunction,
								double xFilesFactor,
								int steps,
								int rows,
								CdpPrepContainer cdpPrepContainer) throws Exception {
	
		this.setConsolidationFunction(consolidationFunction);
		this.setXFilesFactor(xFilesFactor);
		this.setSteps(steps);
		this.setRows(rows);
		this.setCdpPrepContainer(cdpPrepContainer);
	}
	/**
	* Replaces the consolidation function in this archive.
	* @param consolidationFunction the consolidation function of the archive to set
	*/
	public void setConsolidationFunction(String consolidationFunction) throws Exception {
		if (!ConsolidationFunction.isValideCF(consolidationFunction)) 
			throw new Exception("Consolidation Function not match");
		this.consolidationFunction = consolidationFunction.toUpperCase();
	}
	/**
	* Replaces the xFilesFactor in this archive.
	* @param xFilesFactor the xFilesFactor of the archive to set
	*/
	public void setXFilesFactor(double xFilesFactor) {
		this.xFilesFactor = xFilesFactor;
	}
	/**
	* Replaces the steps in this archive.
	* @param steps the steps of the archive to set
	*/
	public void setSteps(int steps) {
		this.steps = steps;
	}
	/**
	* Replaces the rows in this archive.
	* @param rows the rows of the archive to set
	*/
	public void setRows(int rows) {
		this.rows = rows;
	}
	/**
	* Replaces the CdpPrepContainer in this archive.
	* @param cdpPrepContainer the CdpPrepContainer of the archive to set
	*/
	public void setCdpPrepContainer(CdpPrepContainer cdpPrepContainer) {
		this.cdpPrepContainer = cdpPrepContainer;
	}
	/**
	* Returns the consolidation function of this archive.
	*/
	public String getConsolidationFunction() {
		return this.consolidationFunction;
	}
	/**
	* Returns the xfile factor of this archive.
	*/
	public double getXFilesFactor() {
		return this.xFilesFactor;
	}
	/**
	* Returns the steps of this archive.
	*/
	public int getSteps() {
		return this.steps;
	}
	/**
	* Returns the rows of this archive.
	*/
	public int getRows() {
		return this.rows;
	}
	/**
	* Returns the CdpPrepContainer in this archive.
	*/
	public CdpPrepContainer getCdpPrepContainer() {
		return this.cdpPrepContainer;
	}
	/**
	* toString return a string representation of a archive
	*/
	public String toString() {
		return "RRA:"+consolidationFunction+":"+String.valueOf(xFilesFactor)+":"+String.valueOf(steps)+":"+String.valueOf(rows);
	}
}