package rrdService.util;
import rrdService.service.*;
import java.util.*;
import java.io.*;
import java.lang.*;
import java.awt.Color;
/**	
* The class RoundRobinGraphic main purpose is to create graphical representations of the data stored in one or several RRDs. Apart from generating graphs, it can also extract numerical reports.
* @author	Damien Pellier
* @version	1.0
* date : 15/07/2001
* </code>
* @see <a href="RoundRobinGraphic.java">RoundRobinGraphic</a>
* @see Vector
*/
public class RoundRobinGraphic implements Serializable {
	/**
	* Image format of a gif image
	*/
	public static final String GIF_FORMAT = "GIF";
	/**
	* Image format of a png image
	*/
	public static final String PNG_FORMAT = "PNG";
	/**
	* name of graphic
	*/
	protected String graphicName;
	/**
	* The time when the graph should begin.
	*/
	protected Time startTime;
	/**
	* The time when the graph should end.
	*/
	protected Time endTime;
	/**
	* x-grid of this graph
	*/
	protected XGrid xGrid;
	/**
	* y-grid of this graph
	*/
	protected YGrid yGrid;
	/**
	* yGridOption place Y grid dynamically based on graph Y range. Algorithm ensures that you always have grid, that there are enough but not too many grid lines and the grid is metric. That is grid lines are placed every 1, 2, 5 or 10 units. (default autoconfigure)
	*/
	protected Boolean yGridOption;
	/**
	* autoscaleOption compute Y range based on function absolute minimum and maximum values. Default algorithm uses predefined set of ranges. This is good in many cases but it fails miserably when you need to graph something like 260 + 0.001 * sin(x). Default algorithm will use Y range from 250 to 300 and on the graph you will see almost straight line. With --alt-autoscale Y range will be from slightly less the 260 - 0.001 to slightly more then 260 + 0.001 and periodic behavior will be seen. (default autoconfigure)
	*/ 
	protected Boolean autoscaleOption;
	/**
	* autoscaleMaxOption Where --alt-autoscale will modify both the absolute maximum AND minimum values, this option will only affect the maximum value. The minimum value, if not defined on the command line, will be 0. This option can be useful when graphing router traffic when the WAN line uses compression, and thus the throughput may be higher than the WAN line speed. (default autoconfigure)
	*/
	protected Boolean autoscaleMaxOption;
	/**
	* unitsExponent This sets the 10**exponent scaling of the y-axis values. Normally values will be scaled to the appropriate units (k, M, etc.). However you may wish to display units always in k (Kilo, 10e3) even if the data is in the M (Mega, 10e6) range for instance. Value should be an integer which is a multiple of 3 between -18 and 18 inclusive. It is the exponent on the units you which to use. For example, use 3 to display the y-axis values in k (Kilo, 10e3, thousands), use -6 to display the y-axis values in u (Micro, 10e-6, millionths). Use a value of 0 to prevent any scaling of the y-axis values. (default autoconfigure)
	*/
	protected Integer unitsExponent;
	/**
	* verticalLabel vertical label on the left side of the graph. This is normally used to specify the units used. (default autoconfigure)
	*/
	protected String verticalLabel;
	/**
	* widthPixels Width of the drawing area within the graph. This affects the size of the gif. (default 400)
	*/
	protected Integer widthPixels;
	/**
	* heightPixels Width of the drawing area within the graph. This affects the size of the gif. (default 100)
	*/
	protected Integer heightPixels; 
	/**
	* interlacedOption If you set this option, then the resulting GIF will be interlaced. Most web browsers display these incrementally as they load. If you do not use this option, the GIFs default to being progressive scanned. The only effect of this option is to control the format of the GIF on disk. It makes no changes to the layout or contents of the graph. (default false)
	*/
	protected Boolean interlacedOption;
	/**
	* imageInfo After the image has been created, the graph function uses printf together with this format string to create output similar to the PRINT function, only that the printf is supplied with the parameters filename, xsize and ysize. In order to generate an IMG tag suitable for including the graph into a web page, the command line would look like this: example '<IMG SRC="/img/%s" WIDTH="%lu" HEIGHT="%lu" ALT="Demo">'
	*/
	protected String imageInfo;
	/**
	* imageFormat Allows you to produce PNG output from rrdtool service. (default: GIF)
	*/
	protected String imageFormat;
	/**
	* lazyOption Only generate the graph, if the current gif is out of date or not existent. (default false)
	*/
	protected Boolean lazyOption;
	/**
	* upperLimit Defines the value normally located at the upper border of the graph. If the graph contains higher values, the upper border will move upwards to accomodate these values as well. If you want to define an upper-limit which will not move in any event you have to set the --rigid option as well. (default autoconfigure)
	*/
	protected Integer upperLimit;
	/**
	* lowerLimit This is not the lower limit of a graph. But rather, this is the maximum lower bound of a graph. For example, the value -100 will result in a graph that has a lower limit of -100 or less. Use this keyword to expand graphs down. (default autoconfigure)
	*/
	protected Integer lowerLimit;
	/**
	* rigidOption rigid boundaries mode. Normally rrdgraph will automatically expand the lower and upper limit if the graph contains a value outside the valid range. With the r option you can disable this behavior 
	*/
	protected Boolean rigidOption;
	/**
	* base if you are graphing memory (and NOT network traffic) this switch should be set to 1024 so that one Kb is 1024 byte. For traffic measurement, 1 kb/s is 1000 b/s. 
	*/
	protected Integer base;
	/**
	* logarithmicOption logarithmic y-axis scaling 
	*/
	protected Boolean logarithmicOption;
	/**
	* color override the colors for the standard elements of the graph. The COLORTAG must be one of the following symbolic names: BACK ground, CANVAS, SHADEA left/top border, SHADEB right/bottom border, GRID, MGRID major grid, FONT, FRAME and axis of the graph or ARROW. This option can be called multiple times to set several colors. (default colors)
	*/
	protected RoundRobinColor color;
	/**
	* noLegendOption Suppress generation of legend; only render the graph. 
	*/
	protected Boolean noLegendOption;
	/**
	* title Define a title to be written into the graph (default no title)
	*/
	protected String title;
	/**
	* step By default graph calculates the width of one pixle in the time domain and tries to get data at that resolution from the RRD. With this switch you can override this behaviour. If you want rrdgraph to get data at 1 hour resolution from the RRD, then you can set the step to 3600 seconds. Note, that a step smaller than 1 pixle will be silently ignored. (default automatic)
	*/
	protected Integer step;
	/**
	* virtualDataSourceContainer Define virtual name for a data source. This name can then be used in the functions explained below. The DEF call automatically chooses an RRA which contains CF consolidated data in a resolution appropriate for the size of the graph to be drawn. Ideally this means that one data point from the RRA should be represented by one pixel in the graph. If the resolution of the RRA is higher than the resolution of the graph, the data in the RRA will be further consolidated according to the consolidation function ( CF) chosen. 
	*/
	protected VirtualDataSourceContainer virtualDataSourceContainer;
	/**
	* mEVirtualDataSourceContainer Create a new virtual data source by evaluating a mathematical expression, specified in Reverse Polish Notation (RPN). If you have ever used a traditional HP calculator you already know RPN. The idea behind RPN notation is, that you have a stack and push your data onto this stack. When ever you execute an operation, it takes as many data values from the stack as needed. The pushing of data is implicit, so when ever you specify a number or a variable, it gets pushed automatically. If this is all a big load of incomprehensible words for you, maybe an example helps (a more complete explanation is given in [1]): The expression vname+3/2 becomes vname,3,2,/,+ in RPN. First the three values get pushed onto the stack (which now contains (the current value of) vname, a 3 and a 2). Then the / operator pops two values from the stack (3 and 2), divides the first argument by the second (3/2) and pushes the result (1.5) back onto the stack. Then the + operator pops two values (vname and 1.5) from the stack; both values are added up and the result gets pushes back onto the stack. In the end there is only one value left on the stack: The result of the expression.
	*/
	protected MEVirtualDataSourceContainer mEVirtualDataSourceContainer;
	/**
	* graphicPrint allows to print string into the graph below the legend. 
	*/
	protected GraphicPrint graphicPrint;
	/**
	* comment Like GPRINT but the text is simply printed into the graph. 
	*/
	protected String comment;
	/**
	* GraphicObjectContainer fill of the graphic object to print on the graph
	*/	
	protected GraphicObjectContainer graphicObjectContainer;
	/**
	* Default constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	*/
	public RoundRobinGraphic(String graphicName) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param title the title to set into this RoundRobinGraphic
	*/
	public RoundRobinGraphic(String graphicName, String title) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setTitle(title);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param title the title to set into this RoundRobinGraphic
	* @param verticalLabel the verticalLabel to set into this RoundRobinGraphic
	*/
	public RoundRobinGraphic(String graphicName, String title, String verticalLabel) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setTitle(title);
		setVerticalLabel(verticalLabel);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param startTime the startTime to start graphing
	*/
	public RoundRobinGraphic(String graphicName, Time startTime) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setStartTime(startTime);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param startTime the startTime to start graphing
	* @param endTime the endTime to stop graphing
	*/
	public RoundRobinGraphic(String graphicName, Time startTime, Time endTime) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setStartTime(startTime);
		setEndTime(endTime);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param startTime the startTime to start graphing
	* @param endTime the endTime to stop graphing
	* @param verticalLabel the verticalLabel to set into this RoundRobinGraphic
	*/
	public RoundRobinGraphic(String graphicName, Time startTime, Time endTime, String verticalLabel) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setStartTime(startTime);
		setEndTime(endTime);
		setVerticalLabel(verticalLabel);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param title the title to set into this RoundRobinGraphic
	* @param startTime the startTime to start graphing
	* @param endTime the endTime to stop graphing
	*/
	public RoundRobinGraphic(String graphicName, String title, Time startTime, Time endTime) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setTitle(title);
		setStartTime(startTime);
		setEndTime(endTime);
	}
	/**
	* Constructor of RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	* @param title the title to set into this RoundRobinGraphic
	* @param startTime the startTime to start graphing
	* @param endTime the endTime to stop graphing
	* @param verticalLabel the verticalLabel to set into this RoundRobinGraphic
	*/
	public RoundRobinGraphic(String graphicName, String title, Time startTime, Time endTime, String verticalLabel) {
		initRoundRobinGraphic();
		setGraphicName(graphicName);
		setTitle(title);
		setStartTime(startTime);
		setEndTime(endTime);
		setVerticalLabel(verticalLabel);
	}
	/**
	* initRoundRobinGraphic initialize all field of round robin graphic null
	*/
	private void initRoundRobinGraphic() {
		this.graphicName = null;
		this.startTime = null;
		this.endTime = null;
		this.xGrid = null;
		this.yGrid = null;
		this.yGridOption = null;
		this.autoscaleOption = null;
		this.autoscaleMaxOption = null;
		this.unitsExponent = null;
		this.verticalLabel = null;
		this.widthPixels = null;
		this.heightPixels = null;
		this.interlacedOption = null;
		this.imageInfo = null;
		this.imageFormat = null;
		this.lazyOption = null;
		this.upperLimit = null;
		this.lowerLimit = null;
		this.rigidOption = null;
		this.base = null;
		this.logarithmicOption = null;
		this.color = null;
		this.noLegendOption = null;
		this.step = null;
		this.virtualDataSourceContainer = new VirtualDataSourceContainer();
		this.mEVirtualDataSourceContainer = new MEVirtualDataSourceContainer();
		this.graphicPrint = null;
		this.comment = null;
		this.graphicObjectContainer = new GraphicObjectContainer();
	}
	/**
	* Replaces the graphicName of this RoundRobinGraphic
	* @param graphicName the graphicName to set into this RoundRobinGraphic
	*/
	public void setGraphicName(String graphicName) {
		this.graphicName = graphicName;
	}
	/**
	* Replaces the startTime of this RoundRobinGraphic
	* @param startTime the startTime to set into this RoundRobinGraphic
	*/
	public void  setStartTime(Time startTime) {
		this.startTime = startTime;
	}
	/**
	* Replaces the endTime of this RoundRobinGraphic
	* @param endTime the endTime to set into this RoundRobinGraphic
	*/
	public void  setEndTime(Time endTime) {
		this.endTime = endTime;
	}
	/**
	* Replaces the xgrid of this RoundRobinGraphic
	* @param xgrid the xgrid to set into this RoundRobinGraphic
	*/
	public void  setXGrid(XGrid xGrid) {
		this.xGrid = xGrid;
	}	
	/**
	* Replaces the ygrid of this RoundRobinGraphic
	* @param ygrid the ygrid to set into this RoundRobinGraphic
	*/
	public void  setYGrid(YGrid yGrid) {
		this.yGrid = yGrid;
	}
	/**
	* Replaces the yGridOption of this RoundRobinGraphic
	* @param yGridOption the yGridOption to set into this RoundRobinGraphic
	*/
	public void  setYGridOption(boolean yGridOption) {
		this.yGridOption = new Boolean(yGridOption);
	}
	/**
	* Replaces the autoscaleOption of this RoundRobinGraphic
	* @param autoscaleOption the autoscaleOption to set into this RoundRobinGraphic
	*/
	public void  setAutoscaleOption(boolean autoscaleOption) {
		this.autoscaleOption = new Boolean(autoscaleOption);
	}
	/**
	* Replaces the autoscaleMaxOption of this RoundRobinGraphic
	* @param autoscaleMaxOption the autoscaleMaxOption to set into this RoundRobinGraphic
	*/
	public void  setAutoscaleMaxOption(boolean autoscaleMaxOption) {
		this.autoscaleMaxOption = new Boolean(autoscaleMaxOption);
	}
	/**
	* Replaces the unitsExponent of this RoundRobinGraphic
	* @param unitsExponent the unitsExponent to set into this RoundRobinGraphic
	*/
	public void  setUnitsExponent(int unitsExponent) {
		this.unitsExponent = new Integer(unitsExponent);
	}
	/**
	* Replaces the verticalLabel of this RoundRobinGraphic
	* @param verticalLabel the verticalLabel to set into this RoundRobinGraphic
	*/
	public void  setVerticalLabel(String verticalLabel) {
		this.verticalLabel = verticalLabel;
	}
	/**
	* Replaces the widthPixels of this RoundRobinGraphic
	* @param widthPixels the widthPixels to set into this RoundRobinGraphic
	*/
	public void  setWidthPixels(int widthPixels) {
		this.widthPixels = new Integer(widthPixels);
	}
	/**
	* Replaces the heightPixels of this RoundRobinGraphic
	* @param heightPixels the heightPixels to set into this RoundRobinGraphic
	*/
	public void  setHeightPixels(int heightPixels) {
		this.heightPixels = new Integer(heightPixels);
	}
	/**
	* Replaces the interlacedOption of this RoundRobinGraphic
	* @param interlacedOption the interlacedOption to set into this RoundRobinGraphic
	*/
	public void  setInterlacedOption(boolean interlacedOption) {
		this.interlacedOption = new Boolean(interlacedOption);
	}
	/**
	* Replaces the imageInfo of this RoundRobinGraphic
	* @param imageInfo the imageInfo to set into this RoundRobinGraphic
	*/
	public void  setImageInfo(String imageInfo) {
		this.imageInfo = imageInfo;
	}
	/**
	* Replaces the imageFormat of this RoundRobinGraphic
	* @param imageFormat the imageFormat to set into this RoundRobinGraphic
	*/
	public void  setImageFormat(String imageFormat) throws Exception {
		if (!isImageFormat(imageFormat)) throw new Exception("Image format not match GIF_FORMAT or PNG_FORMAT");
		this.imageFormat = imageFormat.toUpperCase();
	}
	/**
	* Replaces the lazyOption of this RoundRobinGraphic
	* @param lazyOption the lazyOption to set into this RoundRobinGraphic
	*/
	public void  setLazyOption(boolean lazyOption) {
		this.lazyOption = new Boolean(lazyOption);
	}
	/**
	* Replaces the upperLimit of this RoundRobinGraphic
	* @param upperLimit the upperLimit to set into this RoundRobinGraphic
	*/
	public void  setUpperLimit(int upperLimit) {
		this.upperLimit = new Integer(upperLimit);
	}
	/**
	* Replaces the lowerLimit of this RoundRobinGraphic
	* @param lowerLimit the lowerLimit to set into this RoundRobinGraphic
	*/
	public void  setLowerLimit(int lowerLimit) {
		this.lowerLimit = new Integer(lowerLimit);
	}
	/**
	* Replaces the rigidOption of this RoundRobinGraphic
	* @param rigidOption the rigidOption to set into this RoundRobinGraphic
	*/
	public void  setRigidOption(boolean rigidOption) {
		this.rigidOption = new Boolean(rigidOption);
	}
	/**
	* Replaces the base of this RoundRobinGraphic
	* @param base the base to set into this RoundRobinGraphic
	*/
	public void  setBase(int base) {
		this.base = new Integer(base);
	}
	/**
	* Replaces the logarithmicOption of this RoundRobinGraphic
	* @param logarithmicOption the logarithmicOption to set into this RoundRobinGraphic
	*/
	public void  setLogarithmicOption(boolean logarithmicOption) {
		this.logarithmicOption = new Boolean(logarithmicOption);
	}
	/**
	* Replaces the color of this RoundRobinGraphic
	* @param color the color to set into this RoundRobinGraphic
	*/
	public void  setColor(RoundRobinColor color) {
		this.color = color;
	}	
	/**
	* Replaces the noLegendOption of this RoundRobinGraphic
	* @param noLegendOption the noLegendOption to set into this RoundRobinGraphic
	*/
	public void  setNoLegendOption(boolean noLegendOption) {
		this.noLegendOption = new Boolean(noLegendOption);
	}
	/**
	* Replaces the title of this RoundRobinGraphic
	* @param title the title to set into this RoundRobinGraphic
	*/
	public void  setTitle(String title) {
		this.title = title;
	}
	/**
	* Replaces the step of this RoundRobinGraphic
	* @param step the step to set into this RoundRobinGraphic
	*/
	public void  setStep(int step) {
		this.step = new Integer(step);
	}
	/**
	* Replaces the VirtualDataSourceontainer of this RoundRobinGraphic
	* @param virtualDataSourceContainer the virtualDataSource to set into this RoundRobinGraphic
	*/
	public void  setVirtualDataSourceontainer(VirtualDataSourceContainer virtualDataSourceContainer) {
		this.virtualDataSourceContainer= virtualDataSourceContainer;
	}
	/**
	* Replaces the MEVirtualDataSourceContainer of this RoundRobinGraphic
	* @param mEVirtualDataSourceContainer the mEVirtualDataSourceContainer to set into this RoundRobinGraphic
	*/
	public void  setMEVirtualDataSource(MEVirtualDataSourceContainer mEVirtualDataSourceContainer) {
		this.mEVirtualDataSourceContainer = mEVirtualDataSourceContainer;
	}
	/**
	* Replaces the graphicPrint of this RoundRobinGraphic
	* @param graphicPrint the graphicPrint to set into this RoundRobinGraphic
	*/
	public void  setGraphicPrint(GraphicPrint graphicPrint) {
		this.graphicPrint = graphicPrint;
	}
	/**
	* Replaces the comment of this RoundRobinGraphic
	* @param comment the comment to set into this RoundRobinGraphic
	*/
	public void  setComment(String comment) {
		this.comment = comment;
	}
	/**
	* Replaces the graphic object container of this RoundRobinGraphic
	* @param graphicObjectContainer the graphic object container of the RoundRobinGraphic
	*/
	public void  setgraphicObjectContainer(GraphicObjectContainer graphicObjectContainer) {
		this.graphicObjectContainer = graphicObjectContainer;
	}
	/**
	* Returns the graphicName of this RoundRobinGraphic
	*/
	public String getGraphicName() {
		return this.graphicName;
	}
	/**
	* Returns the startTime of this RoundRobinGraphic
	*/
	public Time getStartTime() {
		return startTime;
	}
	/**
	* Returns the endTime of this RoundRobinGraphic
	*/
	public Time getEndTime() {
		return endTime;
	}
	/**
	* Returns the xgrid of this RoundRobinGraphic
	*/
	public XGrid getXGrid() {
		return xGrid;
	}
	/**
	* Returns the ygrid of this RoundRobinGraphic
	*/
	public YGrid getYGrid() {
		return yGrid;
	}
	/**
	* Returns the yGridOption of this RoundRobinGraphic
	*/
	public Boolean getYGridOption() {
		return yGridOption;
	}
	/**
	* Returns the autoscaleOption of this RoundRobinGraphic
	*/
	public Boolean getAutoscaleOption() {
		return autoscaleOption;
	}
	/**
	* Returns the autoscaleMaxOption of this RoundRobinGraphic
	*/
	public Boolean getAutoscaleMaxOption() {
		return autoscaleMaxOption;
	}
	/**
	* Returns the unitsExponent of this RoundRobinGraphic
	*/
	public Integer getUnitsExponent() {
		return unitsExponent;
	}
	/**
	* Returns the verticalLabel of this RoundRobinGraphic
	*/
	public String getVerticalLabel() {
		return verticalLabel;
	}
	/**
	* Returns the widthPixels of this RoundRobinGraphic
	*/
	public Integer getWidthPixels() {
		return widthPixels;
	}
	/**
	* Returns the heightPixels of this RoundRobinGraphic
	*/
	public Integer getHeightPixels() {
		return heightPixels;
	}
	/**
	* Returns the interlacedOption of this RoundRobinGraphic
	*/
	public Boolean getInterlacedOption() {
		return interlacedOption;
	}
	/**
	* Returns the imageInfo of this RoundRobinGraphic
	*/
	public String getImageInfo() {
		return imageInfo;
	}
	/**
	* Returns the imageFormat of this RoundRobinGraphic
	*/
	public String getImageFormat() {
		return imageFormat;
	}
	/**
	* Returns the lazyOption of this RoundRobinGraphic
	*/
	public Boolean getLazyOption() {
		return lazyOption;
	}
	/**
	* Returns the upperLimit of this RoundRobinGraphic
	*/
	public Integer getUpperLimit() {
		return upperLimit;
	}
	/**
	* Returns the lowerLimit of this RoundRobinGraphic
	*/
	public Integer getLowerLimit() {
		return lowerLimit;
	}
	/**
	* Returns the rigidOption of this RoundRobinGraphic
	*/
	public Boolean getRigidOption() {
		return rigidOption;
	}
	/**
	* Returns the base of this RoundRobinGraphic
	*/
	public Integer getBase() {
		return base;
	}
	/**
	* Returns the logarithmicOption of this RoundRobinGraphic
	*/
	public Boolean getLogarithmicOption() {
		return logarithmicOption;
	}
	/**
	* Returns the color of this RoundRobinGraphic
	*/
	public RoundRobinColor getColor() {
		return color;
	}
	/**
	* Returns the noLegendOption of this RoundRobinGraphic
	*/
	public Boolean getNoLegendOption() {
		return noLegendOption;
	}
	/**
	* Returns the title of this RoundRobinGraphic
	*/
	public String getTitle() {
		return title;
	}
	/**
	* Returns the step of this RoundRobinGraphic
	*/
	public Integer getStep() {
		return step;
	}
	/**
	* Returns the virtualDataSourceContainer of this RoundRobinGraphic
	*/
	public VirtualDataSourceContainer getVirtualDataSourceConatiner() {
		return virtualDataSourceContainer;
	}
	/**
	* Returns the MEVirtualDataSourceContainer of this RoundRobinGraphic
	*/
	public MEVirtualDataSourceContainer getMEVirtualDataSourceContainer() {
		return mEVirtualDataSourceContainer;
	}
	/**
	* Returns the graphicPrint of this RoundRobinGraphic
	*/
	public GraphicPrint getGraphicPrint() {
		return graphicPrint;
	}
	/**
	* Returns the comment of this RoundRobinGraphic
	*/
	public String getComment() {
		return comment;
	}
	/**
	* Returns the graphic object container of this RoundRobinGraphic
	*/
	public GraphicObjectContainer getGraphicObjectContainer() {
		return graphicObjectContainer;
	}
	/**
	* Add a virtualDataSource into the RoundRobinGraphic
	* @param virtualDataSource the virtualDataSource to add 
	*/
	public void addDEF(VirtualDataSource virtualDataSource) {
		this.virtualDataSourceContainer.addElement(virtualDataSource);
	}
	/**
	* Add a mEVirtualDataSource into the RoundRobinGraphic
	* @param meVirtualDataSource the mEVirtualDataSource to add 
	*/
	public void addCDEF(MathExpressionVirtualDataSource mEVirtualDataSource) {
		this.mEVirtualDataSourceContainer.addElement(mEVirtualDataSource);
	}
	/**
	* Add a graphic Object into the RoundRobinGraphic
	* @param graphicObject the graphicObject to add 
	*/
	public void addGraphicObject(GraphicObject graphicObject) {
		this.graphicObjectContainer.addElement(graphicObject);
	}
	/**
	* isImageFormat returns true if the string parameter is a valide image format
	*/
	public boolean isImageFormat(String imageFormat) {
		return (imageFormat.equalsIgnoreCase(GIF_FORMAT) || imageFormat.equalsIgnoreCase(PNG_FORMAT));
	}
	/**
	* Returns a string representation of this round robin graphic of the rrdtool commande line
	*/
	public String toString() {
		String result = new String();
		try {		
	 		if (startTime !=null) {
	 			result += "--start "+startTime.toString()+" ";
	 		}
	 		if (endTime != null) {
	 			result += "--end "+endTime.toString()+" ";
	 		}
	 		if (xGrid != null) {
	 			result += "--x-grid "+xGrid.toString()+" ";
	 		}
	 		if (yGrid != null) {
	 			result += "--y-grid "+yGrid.toString()+" ";
	 		}
	 		if (yGridOption != null && yGridOption.booleanValue()) {
	 			result += "--alt-y-grid ";
	 		}
	 		if (autoscaleOption != null && autoscaleOption.booleanValue()) {
	 			result += "--alt-autoscale ";
	 		}
	  		if (autoscaleMaxOption != null && autoscaleMaxOption.booleanValue()) {
	  			result += "--alt-autoscale-max ";
	  		}
		  	if (unitsExponent != null) {
		  		result += "--units-exponent "+unitsExponent.toString()+" ";
		  	}
		  	if (verticalLabel !=null) {
		  		result += "--vertical-label "+verticalLabel+" ";
		  	}
		  	if (widthPixels != null) {
		  		result += "--width "+widthPixels.toString()+" ";
		  	}
		  	if (heightPixels != null) {
		  		result += "--height "+heightPixels.toString()+" ";
		  	}
		  	if (interlacedOption != null && interlacedOption.booleanValue()) {
		  		result += "--interlaced ";
		  	}
		  	if (imageInfo != null) {
		  		result += "--imginfo  "+imageInfo+" ";
		  	}
		  	if (imageFormat != null) {
		  		result += "--imgformat "+imageFormat+" ";
		  	}
			if (lazyOption != null && lazyOption.booleanValue()) {
				result += "--lazy ";
			}
			if (upperLimit != null) {
				result += "--upper-limit "+upperLimit.toString()+" ";
			}
			if (lowerLimit != null) {
				result += "--lower-limit "+lowerLimit.toString()+" ";
			}
			if (rigidOption != null && rigidOption.booleanValue()) {
				result += "--rigid ";
			}
			if (base != null) {
				result += "Base = "+base.toString()+"--base ";
			}
			if (logarithmicOption != null && logarithmicOption.booleanValue()) {
				result += "--logarithmic ";
			}
			if (color != null) {
				result += "--color "+color.toString()+" ";
			}
			if (noLegendOption != null && noLegendOption.booleanValue()) {
				result += "--no-legend ";
			}
			if (title != null) {
				result += "--title \""+title+"\" ";
			}
			if (step != null) {
				result += "--step "+step.toString()+" ";
			}
			if (virtualDataSourceContainer.size() != 0) {
				result += virtualDataSourceContainer.toString()+" ";
			}
			if (mEVirtualDataSourceContainer.size() != 0) {
				result += mEVirtualDataSourceContainer.toString()+" ";
			}
			if (graphicPrint != null) {
				result += graphicPrint.toString()+" ";
			}
			if (comment != null) {
				result += "COMMENT:"+comment+" ";
			}
			if (graphicObjectContainer.size() != 0) {
				result += graphicObjectContainer.toString();
			}
		} catch (Exception e) {}
		return result;
	}
	/**
	* Returns a string representation of this round robin graphic to visualize all fields and round robin graphics options
	*/
	public String listRoundRobinGraphic() {
		String result = new String();
		try {
			if (graphicName != null)  {
				result += "Graphic name = "+graphicName+"\n";
			}
			else {
				result += "Graphic name = unknown\n";
			}
	 		if (startTime !=null) {
	 			result += "Start time = "+startTime.toString()+"\n";
	 		}
	 		else {
	 			result += result += "Start time = default\n";
	 		}
	 		if (endTime != null) {
	 			result += "End time = "+endTime.toString()+"\n";
	 		}
	 		else {
	 			result += "End time = default\n";
	 		}
	 		if (xGrid != null) {
	 			result += "XGrid = "+xGrid.toString()+"\n";
	 		}
	 		else {
	 			result += "XGrid = default\n";
	 		}
	 		if (yGrid != null) {
	 			result += "YGrid = "+yGrid.toString()+"\n";
	 		}
	 		else {
	 			result += "YGrid = default\n";
	 		}
	 		if (yGridOption != null) {
	 			result += "YGrid option = "+yGridOption.toString()+"\n";
	 		}
	 		else {
	 			result += "YGrid option = default\n";
	 		}
	 		if (autoscaleOption != null) {
	 			result += "Autoscale option = "+autoscaleOption.toString()+"\n";
	 		}
	 		else {
	 			result += "Autoscale option = default\n";
	 		}
	  		if (autoscaleMaxOption != null) {
	  			result += "Autoscale max option = "+autoscaleMaxOption.toString()+"\n";
	  		}
	  		else {
	  			result += "Autoscale max option = default\n";
	  		}
		  	if (unitsExponent != null) {
		  		result += "Units exponent = "+unitsExponent.toString()+"\n";
		  	}
		  	else {
		  		result += "Units exponent = default\n";
		  	}
		  	if (verticalLabel !=null) {
		  		result += "Vertical label = "+ verticalLabel.toString()+"\n";
		  	}
		  	else {
		  		result += "Vertical label = default\n";
		  	}
		  	if (widthPixels != null) {
		  		result += "Width pixels = "+ widthPixels.toString()+"\n";
		  	}
		  	else {
		  		result += "Width pixels = default\n";
		  	}
		  	if (heightPixels != null) {
		  		result += "Height pixels = "+ heightPixels.toString()+"\n";
		  	}
		  	else {
		  		result += "Height pixels = default\n";
		  	}
		  	if (interlacedOption != null) {
		  		result += "Interlaced option = "+ interlacedOption.toString()+"\n";
		  	}
		  	else {
		  		result += "Interlaced option = default\n";
		  	}
		  	if (imageInfo != null) {
		  		result += "Image info = "+ imageInfo.toString()+"\n";
		  	}
		  	else {
		  		result += "Image info = default\n";
		  	}
		  	if (imageFormat != null) {
		  		result += "Image format = "+ imageFormat.toString()+"\n";
		  	}
		  	else {
		  		result += "Image format = default\n";
		  	}
			if (lazyOption != null) {
				result += "Lazy option = "+ lazyOption.toString()+"\n";
			}
			else {
				result += "Lazy option = default\n";
			}
			if (upperLimit != null) {
				result += "Upper limit = "+ upperLimit.toString()+"\n";
			}
			else {
				result += "Upper limit = default\n";
			}
			if (lowerLimit != null) {
				result += "Lower limit = "+ lowerLimit.toString()+"\n";
			}
			else {
				result += "Lower limit = default\n";
			}
			if (rigidOption != null) {
				result += "Rigid option = "+ rigidOption.toString()+"\n";
			}
			else {
				result += "Rigid option = default\n";
			}
			if (base != null) {
				result += "Base = "+ base.toString()+"\n";
			}
			else {
				result += "Base = default\n";
			}
			if (logarithmicOption != null) {
				result += "Logarithmic option = "+ logarithmicOption.toString()+"\n";
			}
			else {
				result += "Logarithmic option = default\n";
			}
			if (color != null) {
				result += "Color = "+color.toString()+"\n";
			}
			else {
				result += "Color = default\n";
			}
			if (noLegendOption != null) {
				result += "No legend option = "+ noLegendOption.toString()+"\n";
			}
			else {
				result += "No legend option = default\n";
			}
			if (title != null) {
				result += "Title = "+ title.toString()+"\n";
			}
			else {
				result += "Title = default\n";
			}
			if (step != null) {
				result += "Step = "+step.toString()+"\n";
			}
			else {
				result += "Step = default\n";
			}
			if (virtualDataSourceContainer.size() != 0) {
				result += "Virtual data source = \n"+ virtualDataSourceContainer.toString()+"\n";
			}
			else {
				result += "Virtual data source = default\n";
			}
			if (mEVirtualDataSourceContainer.size() != 0) {
				result += "Math expression virtual data source = \n"+ mEVirtualDataSourceContainer.toString()+"\n";
			}
			else {
				result += "Math expression virtual data source = default\n";
			}
			if (graphicPrint != null) {
				result += "Graphic print = "+ graphicPrint.toString()+"\n";
			}
			else {
				result += "Graphic print = default\n";
			}
			if (comment != null) {
				result += "Comment = "+ comment;
			}
			else {
				result += "Comment = default";
			}
			if (graphicObjectContainer.size() != 0) {
				result += "Graphic Object = \n"+graphicObjectContainer.toString();
			}
			else {
				result += "Graphic Object = default";
			}
		} catch (Exception e) {}
		return result;
	}
}	